# Generated by Django 3.1.4 on 2020-12-04 10:54

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ApproveTestModel',
            fields=[
                ('create_dt', models.DateTimeField(blank=True, editable=False, help_text='ModelTimetrackingMixin.create_dt.help_text', null=True, verbose_name='ModelTimetrackingMixin.create_dt.verbose_name')),
                ('update_dt', models.DateTimeField(blank=True, editable=False, help_text='ModelTimetrackingMixin.update_dt.help_text', null=True, verbose_name='ModelTimetrackingMixin.update_dt.verbose_name')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, help_text='BaseApproveModel.pk.help_text', primary_key=True, serialize=False, verbose_name='BaseApproveModel.pk.verbose_name')),
                ('is_draft', models.BooleanField(db_index=True, default=True, editable=False, help_text='BaseApproveModel.is_draft.help_text', verbose_name='BaseApproveModel.is_draft.verbose_name')),
                ('blocked', models.BooleanField(default=False, help_text='BaseApproveModel.blocked.help_text', verbose_name='BaseApproveModel.blocked.verbose_name')),
                ('ready_to_approve', models.BooleanField(blank=True, default=False, help_text='BaseApproveWorkflowModel.ready_to_approve.help_text', verbose_name='BaseApproveWorkflowModel.ready_to_approve.verbose_name')),
                ('name', models.CharField(help_text='This is always needed field', max_length=128, verbose_name='The Name')),
                ('title', models.CharField(blank=True, help_text='This is only needed to approve this entry', max_length=128, verbose_name='A Title')),
                ('approved', models.OneToOneField(editable=False, help_text='BaseApproveModel.approved.help_text', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='draft', to='approve_workflow_test_app.approvetestmodel', verbose_name='BaseApproveModel.approved.verbose_name')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'approve'),
            },
        ),
        migrations.CreateModel(
            name='RelatedApproveTestModel',
            fields=[
                ('create_dt', models.DateTimeField(blank=True, editable=False, help_text='ModelTimetrackingMixin.create_dt.help_text', null=True, verbose_name='ModelTimetrackingMixin.create_dt.verbose_name')),
                ('update_dt', models.DateTimeField(blank=True, editable=False, help_text='ModelTimetrackingMixin.update_dt.help_text', null=True, verbose_name='ModelTimetrackingMixin.update_dt.verbose_name')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, help_text='BaseApproveModel.pk.help_text', primary_key=True, serialize=False, verbose_name='BaseApproveModel.pk.verbose_name')),
                ('is_draft', models.BooleanField(db_index=True, default=True, editable=False, help_text='BaseApproveModel.is_draft.help_text', verbose_name='BaseApproveModel.is_draft.verbose_name')),
                ('blocked', models.BooleanField(default=False, help_text='BaseApproveModel.blocked.help_text', verbose_name='BaseApproveModel.blocked.verbose_name')),
                ('relation_name', models.CharField(help_text='This is always needed field', max_length=128, verbose_name='The Name')),
                ('relation_title', models.CharField(blank=True, help_text='This is only needed to approve this entry', max_length=128, null=True, verbose_name='A Title')),
                ('approved', models.OneToOneField(editable=False, help_text='BaseApproveModel.approved.help_text', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='draft', to='approve_workflow_test_app.relatedapprovetestmodel', verbose_name='BaseApproveModel.approved.verbose_name')),
                ('main_entry', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='relations', to='approve_workflow_test_app.approvetestmodel')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'approve'),
            },
        ),
    ]
