# Generated by Django 4.1.5 on 2023-02-07 15:29

from django.db import migrations, models

import bx_django_utils.translation


class Migration(migrations.Migration):
    dependencies = [
        ("test_app", "0002_storesavemodel"),
    ]

    operations = [
        migrations.CreateModel(
            name="TranslatedModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "translated",
                    bx_django_utils.translation.TranslationField(["de-de", "en-us", "es"]),
                ),
                (
                    "translated_multiline",
                    bx_django_utils.translation.TranslationField(["de-de", "en-us", "es"], blank=True),
                ),
            ],
        ),
    ]
