# Generated by Django 4.1.5 on 2023-02-03 11:42

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="FeatureFlagModel",
            fields=[
                (
                    "create_dt",
                    models.DateTimeField(
                        blank=True,
                        editable=False,
                        help_text="ModelTimetrackingMixin.create_dt.help_text",
                        null=True,
                        verbose_name="ModelTimetrackingMixin.create_dt.verbose_name",
                    ),
                ),
                (
                    "update_dt",
                    models.DateTimeField(
                        blank=True,
                        editable=False,
                        help_text="ModelTimetrackingMixin.update_dt.help_text",
                        null=True,
                        verbose_name="ModelTimetrackingMixin.update_dt.verbose_name",
                    ),
                ),
                (
                    "cache_key",
                    models.CharField(
                        help_text="FeatureFlagModel.cache_key.help_text",
                        max_length=250,
                        primary_key=True,
                        serialize=False,
                        verbose_name="FeatureFlagModel.cache_key.verbose_name",
                    ),
                ),
                (
                    "state",
                    models.SmallIntegerField(
                        choices=[(1, "enabled"), (0, "disabled")],
                        help_text="FeatureFlagModel.state.help_text",
                        verbose_name="FeatureFlagModel.state.verbose_name",
                    ),
                ),
            ],
            options={
                "verbose_name": "FeatureFlagModel.verbose_name",
                "verbose_name_plural": "FeatureFlagModel.verbose_name_plural",
            },
        ),
    ]
